#====================================================================
# Combinatorics Class, version 1.0
# Copyright (C) 2002 Dmitry Rozmanov
#
#====================================================================
# Implemented:
#       Transpositions of list
#       Sets by M from list of length N.
#       Transpositions of sets by M from list of lenght N.
#
#====================================================================

class Combinatorics:

    #--------------------------------------------------------------------
    # returns indices for all m-memeber sets from n-member list
    def C_ind(self, n, m):
        ""

        # check if m bigger than n
        if m > n or m < 0:
            m = n

        #print m, 'out of', n

        indices = []

        ok = 1

        # main loop
        while ok:
            t = []

            # first pass, result list initialisation
            if not indices:
                for i in range(m - 1, -1, -1):
                    t.append(i)

            # main iteration
            else:
                # copy last set to new one
                for i in indices[-1]: t.append(i)

                # for every member of a new set
                for i in range(m):

                    # check if the member points to the last
                    # possible value of array for this member
                    if t[i] < (n - i - 1) :
                        # if not, then point it to the next value
                        t[i] = t[i] + 1

                        # if we increase not the most left member of the set
                        # then we have to reset all the memebrs that more left
                        # then the current one
                        if i > 0:
                            for k in range(i):
                                t[i - k - 1] = t[i - k] + 1
                        break

                    else:
                        # if yes, do nothing except it is a last member of the set
                        if i == (m - 1) and t[i] == (n - m):
                            # if it is the last member, then finish
                            ok = 0

            # if this is not a last pass append combination to results
            if ok: indices.append(t)

            # end of the main loop

        return indices

    #--------------------------------------------------------------------
    # returns values of all m-memeber sets from n-member list
    def C_val(self, source_list, m):
        ""
        # get indices
        inds = self.C_ind(len(source_list), m)

        # get values using indices
        res = []
        for i in inds:
            t = []
            for k in i:
                t.append(source_list[k])
            res.append(t)

        return res

    #--------------------------------------------------------------------
    # helper function for recursion in self.P_ind()
    def P_loop(self, seq):

        # copy source list not to distort source
        t = []
        for i in seq: t.append(i)

        # for every member in the list rotate and recurse for subset
        res = []
        for i in range(len(t)):
            if len(t) > 2:
                # if it is big enough take slice of it and do recursion
                tinds = self.P_loop(t[1:])
            else:
                # if it is only two members small take the second member only
                tinds = [[t[1]]]

            # combine the results of recursion with the first member of the list
            # and append them to the final result
            for k in tinds:
                #print t[0], k
                res.append([t[0]] + k)

            # rotate list
            t = t[1:] + [t[0]]

        return res

    #--------------------------------------------------------------------
    # returns indices of all possible transpositions of n-member list
    def P_ind(self, n):
        ""

        # init the first set of indices
        first = []
        for i in range(n): first.append(i)

        # get the full set of possible indices
        indices = self.P_loop(first)

        return indices

    #--------------------------------------------------------------------
    # returns values of all possible transpositions of n-member list
    def P_val(self, source_list):
        ""
        # get indices
        inds = self.P_ind(len(source_list))

        # get values using indices
        res = []
        for i in inds:
            t = []
            for k in i:
                t.append(source_list[k])
            res.append(t)

        return res

    #--------------------------------------------------------------------
    # returns indices for all transpositions of all m-memeber sets from n-member list
    def A_ind(self, n, m):
        ""
        # find all possible indices sets
        indices1 = self.C_ind(n, m)

        # and find all possible transpositions of them
        indices2 = []
        for i in indices1:
            indices2 = indices2 + self.P_val(i)

        return indices2

    #--------------------------------------------------------------------
    # returns indices for all transpositions of all m-memeber sets from n-member list
    def A_val(self, source_list, m):
        ""
        # get indices
        inds = self.A_ind(len(source_list), m)

        # get values using indices
        res = []
        for i in inds:
            t = []
            for k in i:
                t.append(source_list[k])
            res.append(t)

        return res

#====================================================================
# Let's check it.
import sys
sys.stdout = open('report.txt', 'w')

print "Testing sequence running.\n"

a = [1, 2, 3, 4]
m = 3

cmb = Combinatorics()

print "Sets by %d from list %s." % (m, a)
inds = cmb.C_ind(len(a), m)
vals = cmb.C_val(a, m)

# let us print the results
for i in range(len(inds)):
    print inds[i], 'Values =', vals[i],
    t = 0
    for k in vals[i]:
        t = t + k
    print 'Sum =', t

# and print their count
print '%d %d-member different sets found for %d-member list.' % (len(inds), m, len(a))
print

print "Transpositions of %s." % a
inds = cmb.P_ind(len(a))
vals = cmb.P_val(a)

# let us print the results
for i in range(len(inds)):
    print inds[i], 'Values =', vals[i],
    t = 0
    for k in vals[i]:
        t = t + k
    print 'Sum =', t

# and print their count
print '%d different transpositions found for %d-member list.' % (len(inds), len(a))
print

print "Transpositions of sets by %d from list %s." % (m, a)
inds = cmb.A_ind(len(a), m)
vals = cmb.A_val(a, m)

# let us print the results
for i in range(len(inds)):
    print inds[i], 'Values =', vals[i],
    t = 0
    for k in vals[i]:
        t = t + k
    print 'Sum =', t

# and print their count
print '%d different transpositions of all %d-member sets from %d-member list found.' % (len(inds), m, len(a))
print

print "Testing sequence successfully completed.\n"     

